@@ -26,15 +26,11 @@
 
 """astroid packaging information"""
 
-from typing import Optional
+from pkg_resources import DistributionNotFound, get_distribution
 
-__version__ = "2.5.5"
-# For an official release, use 'alpha_version = False' and 'dev_version = None'
-alpha_version: bool = False  # Release will be an alpha version if True (ex: '1.2.3a6')
-dev_version: Optional[int] = None
+try:
+    __version__ = get_distribution("astroid").version
+except DistributionNotFound:
+    __version__ = "2.5.7+"
 
-if dev_version is not None:
-    if alpha_version:
-        __version__ += f"a{dev_version}"
-    else:
-        __version__ += f".dev{dev_version}"
+version = __version__
