@@ -1,28 +1,31 @@
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
+
 # Copyright (c) 2017-2018 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 David Euresti <github@euresti.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Copyright (c) 2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 """Astroid hooks for typing.py support."""
 import sys
 import typing
 from functools import partial
 
+import astroid
 from astroid import (
     MANAGER,
+    AttributeInferenceError,
+    InferenceError,
     UseInferenceDefault,
+    context,
     extract_node,
     inference_tip,
     node_classes,
     nodes,
-    context,
-    InferenceError,
-    AttributeInferenceError,
 )
-import astroid
 
 PY37 = sys.version_info[:2] >= (3, 7)
 PY39 = sys.version_info[:2] >= (3, 9)
@@ -105,10 +108,10 @@ def looks_like_typing_typevar_or_newtype(node):
     return False
 
 
-def infer_typing_typevar_or_newtype(node, context=None):
+def infer_typing_typevar_or_newtype(node, context_itton=None):
     """Infer a typing.TypeVar(...) or typing.NewType(...) call"""
     try:
-        func = next(node.func.infer(context=context))
+        func = next(node.func.infer(context=context_itton))
     except InferenceError as exc:
         raise UseInferenceDefault from exc
 
@@ -119,16 +122,16 @@ def infer_typing_typevar_or_newtype(node, context=None):
 
     typename = node.args[0].as_string().strip("'")
     node = extract_node(TYPING_TYPE_TEMPLATE.format(typename))
-    return node.infer(context=context)
+    return node.infer(context=context_itton)
 
 
 def _looks_like_typing_subscript(node):
     """Try to figure out if a Subscript node *might* be a typing-related subscript"""
     if isinstance(node, nodes.Name):
         return node.name in TYPING_MEMBERS
-    elif isinstance(node, nodes.Attribute):
+    if isinstance(node, nodes.Attribute):
         return node.attrname in TYPING_MEMBERS
-    elif isinstance(node, nodes.Subscript):
+    if isinstance(node, nodes.Subscript):
         return _looks_like_typing_subscript(node.value)
     return False
 
@@ -169,7 +172,7 @@ def infer_typing_attr(
         ):
             # node.parent.slots is evaluated and cached before the inference tip
             # is first applied. Remove the last result to allow a recalculation of slots
-            cache = getattr(node.parent, "__cache")
+            cache = node.parent.__cache
             if cache.get(node.parent.slots) is not None:
                 del cache[node.parent.slots]
         return iter([value])
@@ -232,8 +235,7 @@ def _forbid_class_getitem_access(node: nodes.ClassDef) -> None:
         """
         if attr == "__class_getitem__":
             raise AttributeInferenceError("__class_getitem__ access is not allowed")
-        else:
-            return origin_func(attr, *args, **kwargs)
+        return origin_func(attr, *args, **kwargs)
 
     try:
         node.getattr("__class_getitem__")
@@ -298,6 +300,46 @@ def infer_typing_alias(
     return iter([class_def])
 
 
+def _looks_like_tuple_alias(node: nodes.Call) -> bool:
+    """Return True if call is for Tuple alias.
+
+    In PY37 and PY38 the call is to '_VariadicGenericAlias' with 'tuple' as
+    first argument. In PY39+ it is replaced by a call to '_TupleType'.
+
+    PY37: Tuple = _VariadicGenericAlias(tuple, (), inst=False, special=True)
+    PY39: Tuple = _TupleType(tuple, -1, inst=False, name='Tuple')
+    """
+    return (
+        isinstance(node, nodes.Call)
+        and isinstance(node.func, nodes.Name)
+        and (
+            not PY39
+            and node.func.name == "_VariadicGenericAlias"
+            and isinstance(node.args[0], nodes.Name)
+            and node.args[0].name == "tuple"
+            or PY39
+            and node.func.name == "_TupleType"
+            and isinstance(node.args[0], nodes.Name)
+            and node.args[0].name == "tuple"
+        )
+    )
+
+
+def infer_tuple_alias(
+    node: nodes.Call, ctx: context.InferenceContext = None
+) -> typing.Iterator[nodes.ClassDef]:
+    """Infer call to tuple alias as new subscriptable class typing.Tuple."""
+    res = next(node.args[0].infer(context=ctx))
+    class_def = nodes.ClassDef(
+        name="Tuple",
+        parent=node.parent,
+    )
+    class_def.postinit(bases=[res], body=[], decorators=None)
+    func_to_add = astroid.extract_node(CLASS_GETITEM_TEMPLATE)
+    class_def.locals["__class_getitem__"] = [func_to_add]
+    return iter([class_def])
+
+
 MANAGER.register_transform(
     nodes.Call,
     inference_tip(infer_typing_typevar_or_newtype),
@@ -316,3 +358,6 @@ if PY37:
     MANAGER.register_transform(
         nodes.Call, inference_tip(infer_typing_alias), _looks_like_typing_alias
     )
+    MANAGER.register_transform(
+        nodes.Call, inference_tip(infer_tuple_alias), _looks_like_tuple_alias
+    )
