@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2009-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Google, Inc.
@@ -14,29 +13,25 @@
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2018 HoverHell <hoverhell@gmail.com>
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020 Vilnis Termanis <vilnis.termanis@iotics.com>
+# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 
 """this module contains a set of functions to handle python protocols for nodes
 where it makes sense.
 """
 
 import collections
+import itertools
 import operator as operator_mod
 
-import itertools
-
-from astroid import Store
-from astroid import arguments
-from astroid import bases
+from astroid import Store, arguments, bases
 from astroid import context as contextmod
-from astroid import exceptions
-from astroid import decorators
-from astroid import node_classes
-from astroid import helpers
-from astroid import nodes
-from astroid import util
+from astroid import decorators, exceptions, helpers, node_classes, nodes, util
 
 raw_building = util.lazy_import("raw_building")
 objects = util.lazy_import("objects")
@@ -498,8 +493,8 @@ def _infer_context_manager(self, mgr, context):
     elif isinstance(inferred, bases.Instance):
         try:
             enter = next(inferred.igetattr("__enter__", context=context))
-        except (exceptions.InferenceError, exceptions.AttributeInferenceError):
-            raise exceptions.InferenceError(node=inferred)
+        except (exceptions.InferenceError, exceptions.AttributeInferenceError) as exc:
+            raise exceptions.InferenceError(node=inferred) from exc
         if not isinstance(enter, bases.BoundMethod):
             raise exceptions.InferenceError(node=enter)
         yield from enter.infer_call_result(self, context)
@@ -608,7 +603,7 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
             A list of indices, where each index specifies what item to fetch from
             the inference results.
     """
-    # pylint: disable=too-many-locals,too-many-branches,too-many-statements
+    # pylint: disable=too-many-locals,too-many-statements
     def _determine_starred_iteration_lookups(starred, target, lookups):
         # Determine the lookups for the rhs of the iteration
         itered = target.itered()
@@ -686,11 +681,10 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
                     continue
 
                 # We're done unpacking.
-                elts = list(elts)
                 packed = nodes.List(
                     ctx=Store, parent=self, lineno=lhs.lineno, col_offset=lhs.col_offset
                 )
-                packed.postinit(elts=elts)
+                packed.postinit(elts=list(elts))
                 yield packed
                 break
 
