@@ -1,17 +1,23 @@
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Google, Inc.
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2015 Ovidiu Sabou <ovidiu@sabou.org>
 # Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020 Becker Awqatty <bawqatty@mide.com>
+# Copyright (c) 2020 Robin Jarry <robin.jarry@6wind.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 
 """this module contains a set of functions to create astroid trees from scratch
 (build_* functions) or from living object (object_build_* functions)
@@ -19,23 +25,21 @@
 
 import builtins
 import inspect
-import logging
 import os
 import sys
 import types
+import warnings
 
-from astroid import bases
-from astroid import manager
-from astroid import node_classes
-from astroid import nodes
-
+from astroid import bases, manager, node_classes, nodes
 
 MANAGER = manager.AstroidManager()
 # the keys of CONST_CLS eg python builtin types
 
 _CONSTANTS = tuple(node_classes.CONST_CLS)
 _BUILTINS = vars(builtins)
-_LOG = logging.getLogger(__name__)
+TYPE_NONE = type(None)
+TYPE_NOTIMPLEMENTED = type(NotImplemented)
+TYPE_ELLIPSIS = type(...)
 
 
 def _io_discrepancy(member):
@@ -119,17 +123,22 @@ def build_class(name, basenames=(), doc=None):
     return node
 
 
-def build_function(name, args=None, defaults=None, doc=None):
+def build_function(name, args=None, posonlyargs=None, defaults=None, doc=None):
     """create and initialize an astroid FunctionDef node"""
-    args, defaults = args or [], defaults or []
+    args, defaults, posonlyargs = args or [], defaults or [], posonlyargs or []
     # first argument is now a list of decorators
     func = nodes.FunctionDef(name, doc)
     func.args = argsnode = nodes.Arguments()
     argsnode.args = []
+    argsnode.posonlyargs = []
     for arg in args:
         argsnode.args.append(nodes.Name())
         argsnode.args[-1].name = arg
         argsnode.args[-1].parent = argsnode
+    for arg in posonlyargs:
+        argsnode.posonlyargs.append(nodes.Name())
+        argsnode.posonlyargs[-1].name = arg
+        argsnode.posonlyargs[-1].parent = argsnode
     argsnode.defaults = []
     for default in defaults:
         argsnode.defaults.append(nodes.const_factory(default))
@@ -174,14 +183,27 @@ def object_build_class(node, member, localname):
 
 def object_build_function(node, member, localname):
     """create astroid for a living function object"""
-    # pylint: disable=deprecated-method; completely removed in 2.0
-    args, varargs, varkw, defaults = inspect.getargspec(member)
-    if varargs is not None:
-        args.append(varargs)
-    if varkw is not None:
-        args.append(varkw)
+    signature = inspect.signature(member)
+    args = []
+    defaults = []
+    posonlyargs = []
+    for param_name, param in signature.parameters.items():
+        if param.kind == inspect.Parameter.POSITIONAL_ONLY:
+            posonlyargs.append(param_name)
+        elif param.kind == inspect.Parameter.POSITIONAL_OR_KEYWORD:
+            args.append(param_name)
+        elif param.kind == inspect.Parameter.VAR_POSITIONAL:
+            args.append(param_name)
+        elif param.kind == inspect.Parameter.VAR_KEYWORD:
+            args.append(param_name)
+        if param.default is not inspect._empty:
+            defaults.append(param.default)
     func = build_function(
-        getattr(member, "__name__", None) or localname, args, defaults, member.__doc__
+        getattr(member, "__name__", None) or localname,
+        args,
+        posonlyargs,
+        defaults,
+        member.__doc__,
     )
     node.add_local_node(func, localname)
 
@@ -224,7 +246,7 @@ def _base_class_object_build(node, member, basenames, name=None, localname=None)
             instdict = member().__dict__
         else:
             raise TypeError
-    except:  # pylint: disable=bare-except
+    except TypeError:
         pass
     else:
         for item_name, obj in instdict.items():
@@ -254,6 +276,13 @@ def _build_from_function(node, name, member, module):
         object_build_function(node, member, name)
 
 
+def _safe_has_attribute(obj, member):
+    try:
+        return hasattr(obj, member)
+    except Exception:  # pylint: disable=broad-except
+        return False
+
+
 class InspectBuilder:
     """class for building nodes from living object
 
@@ -288,15 +317,17 @@ class InspectBuilder:
 
     def object_build(self, node, obj):
         """recursive method which create a partial ast from real objects
-         (only function, class, and method are handled)
+        (only function, class, and method are handled)
         """
         if obj in self._done:
             return self._done[obj]
         self._done[obj] = node
         for name in dir(obj):
             try:
-                member = getattr(obj, name)
-            except AttributeError:
+                with warnings.catch_warnings():
+                    warnings.filterwarnings("error")
+                    member = getattr(obj, name)
+            except (AttributeError, DeprecationWarning):
                 # damned ExtensionClass.Base, I know you're there !
                 attach_dummy_node(node, name)
                 continue
@@ -335,6 +366,11 @@ class InspectBuilder:
                 # This should be called for Jython, where some builtin
                 # methods aren't caught by isbuiltin branch.
                 _build_from_function(node, name, member, self._module)
+            elif _safe_has_attribute(member, "__all__"):
+                module = build_module(name)
+                _attach_local_node(node, module, name)
+                # recursion
+                self.object_build(module, member)
             else:
                 # create an empty node so that the name is actually defined
                 attach_dummy_node(node, name, member)
@@ -347,10 +383,7 @@ class InspectBuilder:
         # (see http://www.logilab.org/ticket/57299 for instance)
         try:
             modname = getattr(member, "__module__", None)
-        except:  # pylint: disable=bare-except
-            _LOG.exception(
-                "unexpected error while building " "astroid from living object"
-            )
+        except TypeError:
             modname = None
         if modname is None:
             if name in ("__new__", "__subclasshook__"):
@@ -381,8 +414,9 @@ class InspectBuilder:
 
 _CONST_PROXY = {}
 
-# TODO : find a nicer way to handle this situation;
+
 def _set_proxied(const):
+    # TODO : find a nicer way to handle this situation;
     return _CONST_PROXY[const.value.__class__]
 
 
@@ -393,14 +427,16 @@ def _astroid_bootstrapping():
     builder = InspectBuilder()
     astroid_builtin = builder.inspect_build(builtins)
 
-    # pylint: disable=redefined-outer-name
     for cls, node_cls in node_classes.CONST_CLS.items():
-        if cls is type(None):
+        if cls is TYPE_NONE:
             proxy = build_class("NoneType")
             proxy.parent = astroid_builtin
-        elif cls is type(NotImplemented):
+        elif cls is TYPE_NOTIMPLEMENTED:
             proxy = build_class("NotImplementedType")
             proxy.parent = astroid_builtin
+        elif cls is TYPE_ELLIPSIS:
+            proxy = build_class("Ellipsis")
+            proxy.parent = astroid_builtin
         else:
             proxy = astroid_builtin.getattr(cls.__name__)[0]
         if cls in (dict, list, set, tuple):
@@ -419,7 +455,6 @@ def _astroid_bootstrapping():
     builder.object_build(bases.Generator._proxied, types.GeneratorType)
 
     if hasattr(types, "AsyncGeneratorType"):
-        # pylint: disable=no-member; AsyncGeneratorType
         _AsyncGeneratorType = nodes.ClassDef(
             types.AsyncGeneratorType.__name__, types.AsyncGeneratorType.__doc__
         )
@@ -430,8 +465,8 @@ def _astroid_bootstrapping():
         types.GetSetDescriptorType,
         types.GeneratorType,
         types.MemberDescriptorType,
-        type(None),
-        type(NotImplemented),
+        TYPE_NONE,
+        TYPE_NOTIMPLEMENTED,
         types.FunctionType,
         types.MethodType,
         types.BuiltinFunctionType,
