@@ -18,8 +18,8 @@
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 # Copyright (c) 2019 Zbigniew Jędrzejewski-Szmek <zbyszek@in.waw.pl>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Federico Bond <federicobond@gmail.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Federico Bond <federicobond@gmail.com>
 # Copyright (c) 2021 hippo91 <guillaume.peillex@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
@@ -53,6 +53,7 @@ REDIRECT = {
 }
 PY37 = sys.version_info >= (3, 7)
 PY38 = sys.version_info >= (3, 8)
+PY39 = sys.version_info >= (3, 9)
 
 
 def _visit_or_none(node, attr, visitor, parent, visit="visit", **kws):
@@ -742,9 +743,14 @@ class TreeRebuilder:
         newnode.postinit(self.visit(node.value, newnode))
         return newnode
 
-    def visit_keyword(self, node, parent):
+    def visit_keyword(self, node: "ast.keyword", parent: NodeNG) -> nodes.Keyword:
         """visit a Keyword node by returning a fresh instance of it"""
-        newnode = nodes.Keyword(node.arg, parent=parent)
+        if PY39:
+            newnode = nodes.Keyword(
+                node.arg, node.lineno, node.col_offset, parent=parent
+            )
+        else:
+            newnode = nodes.Keyword(node.arg, parent=parent)
         newnode.postinit(self.visit(node.value, newnode))
         return newnode
 
@@ -1064,10 +1070,18 @@ class TreeRebuilder:
         self, node: "ast.MatchMapping", parent: NodeNG
     ) -> nodes.MatchMapping:
         newnode = nodes.MatchMapping(node.lineno, node.col_offset, parent)
+        name_node: Optional[nodes.AssignName] = None
+        if node.rest is not None:
+            # Add AssignName node for 'node.rest'
+            # https://bugs.python.org/issue43994
+            name_node = nodes.AssignName(
+                node.rest, node.lineno, node.col_offset, newnode
+            )
+            self._save_assignment(name_node)
         newnode.postinit(
             keys=[self.visit(child, newnode) for child in node.keys],
             patterns=[self.visit(pattern, newnode) for pattern in node.patterns],
-            rest=node.rest,
+            rest=name_node,
         )
         return newnode
 
@@ -1086,18 +1100,36 @@ class TreeRebuilder:
         return newnode
 
     def visit_matchstar(self, node: "ast.MatchStar", parent: NodeNG) -> nodes.MatchStar:
-        return nodes.MatchStar(node.lineno, node.col_offset, parent, name=node.name)
+        newnode = nodes.MatchStar(node.lineno, node.col_offset, parent)
+        name_node: Optional[nodes.AssignName] = None
+        if node.name is not None:
+            # Add AssignName node for 'node.name'
+            # https://bugs.python.org/issue43994
+            name_node = nodes.AssignName(
+                node.name, node.lineno, node.col_offset, newnode
+            )
+            self._save_assignment(name_node)
+        newnode.postinit(name=name_node)
+        return newnode
 
     def visit_matchas(self, node: "ast.MatchAs", parent: NodeNG) -> nodes.MatchAs:
-        newnode = nodes.MatchAs(None, None, parent)
+        newnode = nodes.MatchAs(node.lineno, node.col_offset, parent)
+        name_node: Optional[nodes.AssignName] = None
+        if node.name is not None:
+            # Add AssignName node for 'node.name'
+            # https://bugs.python.org/issue43994
+            name_node = nodes.AssignName(
+                node.name, node.lineno, node.col_offset, newnode
+            )
+            self._save_assignment(name_node)
         newnode.postinit(
             pattern=_visit_or_none(node, "pattern", self, newnode),
-            name=node.name,
+            name=name_node,
         )
         return newnode
 
     def visit_matchor(self, node: "ast.MatchOr", parent: NodeNG) -> nodes.MatchOr:
-        newnode = nodes.MatchOr(None, None, parent)
+        newnode = nodes.MatchOr(node.lineno, node.col_offset, parent)
         newnode.postinit(
             patterns=[self.visit(pattern, newnode) for pattern in node.patterns]
         )
